/****************************************************************************

    PROGRAM: Generic.c

    PURPOSE: Generic template for Windows applications

    FUNCTIONS:

        WinMain() - calls initialization function, processes message loop
        InitApplication() - initializes window data and registers window
        InitInstance() - saves instance handle and creates main window
        MainWndProc() - processes messages
        About() - processes messages for "About" dialog box

    COMMENTS:

        Windows can have several copies of your application running at the
        same time.  The variable hInst keeps track of which instance this
        application is so that processing will be to the correct window.

****************************************************************************/

#include "windows.h"                /* required for all Windows applications */
#include "generic.h"                /* specific to this program              */

HANDLE hInst;                       /* current instance                      */
HWND hwndListFrom, hwndListTo;
HWND hwndFrom, hwndTo;
FARPROC lpListProc;
HCURSOR hCursorDrag;
HCURSOR hCursorNoDrop;

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point 
        for the program.  This function calls the application initialization 
        routine, if no other instance of the program is running, and always 
        calls the instance initialization routine.  It then executes a message 
        retrieval and dispatch loop that is the top-level control structure 
        for the remainder of execution.  The loop is terminated when a WM_QUIT 
        message is received, at which time this function exits the application 
        instance by returning the value passed by PostQuitMessage(). 

        If this function must abort before entering the message loop, it 
        returns the conventional value NULL.  

****************************************************************************/

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                            /* current instance             */
HANDLE hPrevInstance;                        /* previous instance            */
LPSTR lpCmdLine;                             /* command line                 */
int nCmdShow;                                /* show-window type (open/icon) */
{
    MSG msg;                                 /* message                      */

    if (!hPrevInstance)                  /* Other instances of app running? */
        if (!InitApplication(hInstance)) /* Initialize shared things */
            return (FALSE);              /* Exits if unable to initialize     */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage(&msg,        /* message structure                      */
            NULL,                  /* handle of window receiving the message */
            NULL,                  /* lowest message to examine              */
            NULL))                 /* highest message to examine             */
        {
        TranslateMessage(&msg);    /* Translates virtual key codes           */
        DispatchMessage(&msg);     /* Dispatches message to window           */
    }
    return (msg.wParam);           /* Returns the value from PostQuitMessage */
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other 
        instances of the application are running.  This function performs 
        initialization tasks that can be done once for any number of running 
        instances.  

        In this case, we initialize a window class by filling out a data 
        structure of type WNDCLASS and calling the Windows RegisterClass() 
        function.  Since all instances of this application use the same window 
        class, we only need to do this when the first instance is initialized.  


****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                              /* current instance           */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style = NULL;                    /* Class style(s).                    */
    wc.lpfnWndProc = MainWndProc;       /* Function to retrieve messages for  */
                                        /* windows of this class.             */
    wc.cbClsExtra = 0;                  /* No per-class extra data.           */
    wc.cbWndExtra = 0;                  /* No per-window extra data.          */
    wc.hInstance = hInstance;           /* Application that owns the class.   */
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = COLOR_WINDOW+1; 
    wc.lpszMenuName =  "GenericMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "GenericWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class */
    
    if (RegisterClass(&wc))
    {
       wc.style = 0;                    /* Class style(s).                    */
       wc.lpfnWndProc = PopupWndProc;      /* Function to retrieve messages for  */
                                           /* windows of this class.             */
       wc.cbClsExtra = 0;                  /* No per-class extra data.           */
       wc.cbWndExtra = 0;                  /* No per-window extra data.          */
       wc.hInstance = hInstance;           /* Application that owns the class.   */
       wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
       wc.hCursor = LoadCursor(NULL, IDC_ARROW);
       wc.hbrBackground = COLOR_WINDOW+1; 
       wc.lpszMenuName =  NULL;   
       wc.lpszClassName = "PopupClass";    /* Name used in call to CreateWindow. */
       
       return (RegisterClass(&wc));
    }
    else return FALSE;

}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

        This function is called at initialization time for every instance of 
        this application.  This function performs initialization tasks that 
        cannot be shared by multiple instances.  

        In this case, we save the instance handle in a static variable and 
        create and display the main program window.  
        
****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
    HANDLE          hInstance;          /* Current instance identifier.       */
    int             nCmdShow;           /* Param for first ShowWindow() call. */
{
    HWND            hWnd;               /* Main window handle.                */

    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hWnd = CreateWindow(
        "GenericWClass",                /* See RegisterClass() call.          */
        "Generic Sample Application",   /* Text for window title bar.         */
        WS_OVERLAPPEDWINDOW,            /* Window style.                      */
        CW_USEDEFAULT,                  /* Default horizontal position.       */
        CW_USEDEFAULT,                  /* Default vertical position.         */
        CW_USEDEFAULT,                  /* Default width.                     */
        CW_USEDEFAULT,                  /* Default height.                    */
        NULL,                           /* Overlapped windows have no parent. */
        NULL,                           /* Use the window class menu.         */
        hInstance,                      /* This instance owns this window.    */
        NULL                            /* Pointer not needed.                */
    );

    /* If window could not be created, return "failure" */

    if (!hWnd)
        return (FALSE);

    /* Make the window visible; update its client area; and return "success" */

    ShowWindow(hWnd, nCmdShow);  /* Show the window                        */
    UpdateWindow(hWnd);          /* Sends WM_PAINT message                 */
    return (TRUE);               /* Returns the value from PostQuitMessage */

}

//****************************************************************************
// Function: MainWndProc
//
// Purpose: Message handler for main overlapped window.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author              Reason
//           6/18/92    VC       Created
//****************************************************************************
long FAR PASCAL MainWndProc(HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{
    FARPROC lpProcAbout;                  // pointer to the About dialog function 
    static FARPROC lpSubClassProc;        // pointer to list box subclass procedure

    switch (message) {
        case WM_CREATE:
           hwndFrom = CreateWindow( "PopupClass", "Drag Source",  
                             WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_BORDER,            
                             50, 50, 250, 150,
                             hWnd,         
                             NULL,      
                             hInst,                      
                             NULL);
           
           hwndListFrom = CreateWindow( "LISTBOX", "",  
                             WS_CHILD | WS_VISIBLE | WS_BORDER | WS_HSCROLL,            
                             10, 10, 200, 100,
                             hwndFrom,         
                             NULL,      
                             hInst,                      
                             NULL);
           
           // Subclass the list box in the 'Drag Source' window
           lpSubClassProc = MakeProcInstance((FARPROC)SubClassProc, hInst);
           lpListProc = (FARPROC)SetWindowLong(hwndListFrom, GWL_WNDPROC, (LONG)lpSubClassProc);
           
           // Add items to the list in the 'Drag Source' window
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"First");
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Second");
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Third");
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Fourth");
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Fifth");
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Sixth");
           SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Seventh");
           
           hwndTo = CreateWindow( "PopupClass", "Drag Destination",  
                             WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_BORDER,            
                             350, 50, 250, 150,
                             hWnd,         
                             NULL,      
                             hInst,                      
                             NULL);
                             
           hwndListTo = CreateWindow( "LISTBOX", "",  
                             WS_CHILD | WS_VISIBLE |  WS_BORDER | WS_HSCROLL,            
                             10, 10, 200, 100,
                             hwndTo,         
                             NULL,      
                             hInst,                      
                             NULL);
           
           // Load cursors to be used during the drag
           hCursorDrag = LoadCursor(hInst, "DragCursor");
           hCursorNoDrop = LoadCursor(hInst, "NoDropCursor");
                                   
           break;
           
          
        case WM_COMMAND:      
            if (wParam == IDM_ABOUT) 
            {
                lpProcAbout = MakeProcInstance(About, hInst);
                DialogBox(hInst, "AboutBox", hWnd, lpProcAbout); 
                FreeProcInstance(lpProcAbout);
                break;
            }
            else                         
                return (DefWindowProc(hWnd, message, wParam, lParam));

        case WM_CLOSE:                
            // Restore the original list box procedure
            SetWindowLong(hwndListFrom, GWL_WNDPROC, (LONG) lpListProc);
            return (DefWindowProc(hWnd, message, wParam, lParam));
            break;
        
        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:                         
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (NULL);
}

//****************************************************************************
// Function: PopupWndProc
//
// Purpose: Message handler for popup windows that contain the list.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author              Reason
//           2/17/92    Vinoo Cherian       Created
//****************************************************************************
long FAR PASCAL PopupWndProc(HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{
   switch (message)
   {
      default:
         return (DefWindowProc(hWnd, message, wParam, lParam));
   }
   return FALSE;         
}

//****************************************************************************
// Function: SubClassProc
//
// Purpose: Subclass procedure for the list box.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author              Reason
//           2/17/92    Vinoo Cherian       Created
//****************************************************************************
long FAR PASCAL SubClassProc(HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{
   static BOOL bTracking = FALSE;      // Indicated if we are tracking the mouse in the drag loop
   static BOOL bDrag = FALSE;          // Indicated if the drag has started (if the user has
                                       // moved the mouse outside the drag rectangle).
   static HCURSOR hCursorOld = NULL;
   
   switch (message)
   {
      case WM_CANCELMODE:
         // WM_CANCELMODE is sent to the window that has captured the mouse before
         // a message box or modal dialog is displayed. If we were dragging the item
         // cancel the drag.
         bTracking = FALSE;
         ReleaseCapture();
         if (bDrag)
            SetCursor(hCursorOld);
            
         break;
         
      case WM_LBUTTONDOWN:
      {
         BOOL bDragSuccess = FALSE;  // Was the list box item dragged into the destination         
         MSG msg;
         char lpszDragItem[30];      // Text of item being dragged
         int nDragItem,              // Index of item that is being dragged
             nCXFrame, nCYFrame;     // Dimensions of a thick frame. Used to construct 
                                     //  a rectangle to determine if the user started
                                     //  dragging an item or merely clicked on it.  
         RECT rectIsDrag;            // Rectangle to determine if dragging has started.                       
         POINT pt;
         HWND hwndWFP;               // Window handle returned by WindowFromPoint().
              

         // Pass the WM_LBUTTONDOWN to the list box window procedure. Then
         // fake a WM_LBUTTONUP so that we can track the drag.
         CallWindowProc(lpListProc, hWnd, message, wParam, lParam);
         CallWindowProc(lpListProc, hWnd, WM_LBUTTONUP, wParam, lParam);
         

         // Create a tiny rectangle to determine if item was dragged or merely clicked on.
         // If the user moves outside this rectangle we assume that the dragging has
         // started.
         pt.x = LOWORD(lParam);
         pt.y = HIWORD(lParam);                  
         nCXFrame = GetSystemMetrics(SM_CXFRAME);
         nCYFrame = GetSystemMetrics(SM_CYFRAME);
         SetRect(&rectIsDrag,  pt.x - (nCXFrame<<1), pt.y - (nCYFrame<<1),
                               pt.x + (nCXFrame<<1), pt.y + (nCYFrame<<1));
                               
         bTracking = TRUE;       // Track the drag until bTracking is set to FALSE
         
         // Capture the mouse and determine which item was selected in the source list                      
         SetCapture(hWnd);
         nDragItem = SendMessage(hwndListFrom, LB_GETCURSEL, 0, 0);
         SendMessage(hwndListFrom, LB_GETTEXT, nDragItem, (LONG)lpszDragItem);
         
         // Drag loop                      
         while (bTracking)
         {  
            // Retrieve mouse messages and keyboard messages. We retrieve keyboard
            // messages so that the system queue is not filled by keyboard messages
            // during the drag (This can happen if the user madly types while dragging!)
            // If neither mouse nor keyboard messages are available we wait. Both
            // PeekMessage() and Waitmessage() will yield to other apps.                             
            while (!PeekMessage(&msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE)
                   && !PeekMessage(&msg, NULL, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE))
               WaitMessage();      
           
            switch(msg.message)
            {
               case WM_MOUSEMOVE:
                  if (!bDrag)
                  {
                     // Check if the user has moved out of the Drag rect. This indicates
                     // that the drag has started.
                     if (!PtInRect(&rectIsDrag, pt))
                     {
                        hCursorOld = SetCursor(hCursorDrag);
                        bDrag = TRUE;     // Drag has started
                     }
                  }
                  
                  pt.x = LOWORD(msg.lParam);
                  pt.y = HIWORD(msg.lParam);
                  
                  if (bDrag)
                  {  
                     // Change the cursor depending on where the mouse is to provide
                     // a visual indication of where the item is to be dropped.
                     ClientToScreen(hWnd, &pt);
                     hwndWFP = WindowFromPoint(pt);
                     if (hwndWFP == hwndFrom || hwndWFP == hwndListFrom || 
                           hwndWFP == hwndTo || hwndWFP == hwndListTo)     
                        SetCursor(hCursorDrag);
                     else SetCursor(hCursorNoDrop);
                  }

                  break;
                  
               
               case WM_LBUTTONUP: 
                  // End of Drag                             
                  pt.x = LOWORD(msg.lParam);
                  pt.y = HIWORD(msg.lParam);
                  
                  bTracking = FALSE;
                  ClientToScreen(hWnd, &pt);
                  hwndWFP = WindowFromPoint(pt);
                  if (hwndWFP == hwndTo || hwndWFP == hwndListTo)                 
                    bDragSuccess = TRUE;
                  break;
             
               default:
                  // Process the keyboard messages
                  TranslateMessage(&msg);
                  DispatchMessage(&msg);
            }
         }

         ReleaseCapture();
         if (bDrag)
         {
            SetCursor(hCursorOld);
            // Add item that was dragged to the destination list
            if (bDragSuccess)
               SendMessage(hwndListTo, LB_ADDSTRING, 0, (LONG)(LPSTR)lpszDragItem);
            bDrag = FALSE;
         }

         break; 
      }
         
      default:
         return (CallWindowProc(lpListProc, hWnd, message, wParam, lParam));
   }
   return FALSE;         
}

/****************************************************************************

    FUNCTION: About(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

    MESSAGES:

        WM_INITDIALOG - initialize dialog box
        WM_COMMAND    - Input received

    COMMENTS:

        No initialization is needed for this particular dialog box, but TRUE
        must be returned to Windows.

        Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{
    switch (message) {
        case WM_INITDIALOG:                /* message: initialize dialog box */
            return (TRUE);

        case WM_COMMAND:                      /* message: received a command */
            if (wParam == IDOK                /* "OK" box selected?          */
                || wParam == IDCANCEL) {      /* System menu close command? */
                EndDialog(hDlg, TRUE);        /* Exits the dialog box        */
                return (TRUE);
            }
            break;
    }
    return (FALSE);                           /* Didn't process a message    */
}
